#![cfg_attr(docsrs, feature(doc_cfg))]
#![deny(clippy::all)]
#![allow(clippy::ptr_arg)]

use swc_atoms::JsWord;
use swc_common::Span;
use swc_visit::define;
use swc_xml_ast::*;

/// Visitable nodes.
pub trait Node {}

define!({
    pub struct Document {
        pub span: Span,
        pub children: Vec<Child>,
    }

    pub enum DocumentMode {
        NoQuirks,
        LimitedQuirks,
        Quirks,
    }

    pub enum Child {
        DocumentType(DocumentType),
        Element(Element),
        Text(Text),
        CdataSection(CdataSection),
        Comment(Comment),
        ProcessingInstruction(ProcessingInstruction),
    }

    pub struct DocumentType {
        pub span: Span,
        pub name: Option<JsWord>,
        pub public_id: Option<JsWord>,
        pub system_id: Option<JsWord>,
        pub raw: Option<JsWord>,
    }

    pub enum Namespace {
        HTML,
        MATHML,
        SVG,
        XLINK,
        XML,
        XMLNS,
    }

    pub struct Element {
        pub span: Span,
        pub tag_name: JsWord,
        pub attributes: Vec<Attribute>,
        pub children: Vec<Child>,
    }

    pub struct Attribute {
        pub span: Span,
        pub namespace: Option<Namespace>,
        pub prefix: Option<JsWord>,
        pub name: JsWord,
        pub raw_name: Option<JsWord>,
        pub value: Option<JsWord>,
        pub raw_value: Option<JsWord>,
    }

    pub struct Text {
        pub span: Span,
        pub data: JsWord,
        pub raw: Option<JsWord>,
    }

    pub struct CdataSection {
        pub span: Span,
        pub data: JsWord,
        pub raw: Option<JsWord>,
    }

    pub struct ProcessingInstruction {
        pub span: Span,
        pub target: JsWord,
        pub data: JsWord,
    }

    pub struct Comment {
        pub span: Span,
        pub data: JsWord,
        pub raw: Option<JsWord>,
    }
});

impl<T: ?Sized> Node for T {}
